/************************************************************************
 *   IRC - Internet Relay Chat, admin/m_del.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ccr.h"
#include "channel.h"
#include "class.h"
#include "client.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_log.h"
#include "send.h"
#include "struct.h"
#include "umodes.h"
#include "memory.h"
#include "msg.h"
#include "handlers.h"
#include "database/database.h"
#include "varparse.h"
#include "flagparse.h"

int m_del(struct Client *, struct Client *, int, char **);

struct Message del_msgtab = {
  "DEL", 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_del, m_del},
  NULL, NULL
};

var_handler_func m_del_ccr_allow, m_del_ccr_deny;

static struct variable_tier del_ccr_parse_tree[] = {
  {"ALLOW", 0, NULL, m_del_ccr_allow},
  {"DENY",  0, NULL, m_del_ccr_deny},
  {NULL, 0, NULL, NULL}
};

static struct variable_tier del_parse_tree[] = {
  {"CCR", 0, del_ccr_parse_tree, NULL},
  {NULL, 0, NULL, NULL}
};

static struct flag_def ccr_flags[] = {
  {"no-tilde", CCONNECT_FLAGS_NO_TILDE},
  {"need-ident", CCONNECT_FLAGS_NEED_IDENT},
  {"allow-bots", CCONNECT_FLAGS_ALLOW_BOTS},
  {NULL, NULL_FLAG}
};

int
m_del(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  return variable_parse(cptr, sptr, parc, parv, del_parse_tree, "DEL");
}

/* NOTE: deny ccrs take effect immediately. allow ccrs take effect on
 *  the next connection. this may change if absolutely necessary, but for
 *  now I don't want to write that horribly complex routine.
 *  -- asuffield
 */

int
m_del_ccr_allow(struct Client *cptr, struct Client *sptr, int parc, char *parv[], char *varparv[])
{
  if (parc < 3)
    {
      if (!IsServer(sptr))
	sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
		   me.name, parv[0], "DEL CCR.ALLOW");
      return 0;
    }

  switch(delete_allow_ccr(sptr, parv[2]))
    {
    case 0:
      if (MyClient(sptr))
	sendto_one(sptr, ":%s NOTICE %s :Deleted %s from allow ccrs", me.name, sptr->name, parv[2]);
      break;
    case 1:
      if (MyClient(sptr))
	sendto_one(sptr, ":%s NOTICE %s :Couldn't find %s in allow ccrs", me.name, sptr->name, parv[2]);
      break;
    }

  sendto_serv_butone(cptr, ":%s DEL CCR.ALLOW %s", sptr->name, parv[2]);

  return 0;
}

int
m_del_ccr_deny(struct Client *cptr, struct Client *sptr, int parc, char *parv[], char *varparv[])
{
  if (parc < 3)
    {
      if (!IsServer(sptr))
	sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
		   me.name, parv[0], "DEL CCR.DENY");
      return 0;
    }

  switch(delete_deny_ccr(sptr, parv[2]))
    {
    case 0:
      if (MyClient(sptr))
	sendto_one(sptr, ":%s NOTICE %s :Deleted %s from deny ccrs", me.name, sptr->name, parv[2]);
      break;
    case 1:
      if (MyClient(sptr))
	sendto_one(sptr, ":%s NOTICE %s :Couldn't find %s in deny ccrs", me.name, sptr->name, parv[2]);
      break;
    }

  sendto_serv_butone(cptr, ":%s DEL CCR.DENY %s", sptr->name, parv[2]);

  return 0;
}
